import os
import json
import logging
import argparse
from rich.logging import RichHandler


class Version:
    def __init__(self):
        """
        Initialize the Version class.

        Retrieves version components from the data() function and assigns them to instance variables.
        """
        self.major = data()["program"]["version"]["major"]
        self.minor = data()["program"]["version"]["minor"]
        self.patch = data()["program"]["version"]["patch"]
        self.suffix = data()["program"]["version"]["suffix"]

    def full_version(self) -> str:
        """
        Return the full version string composed of the version components.

        :return: The complete version string in the format "major.minor.patchsuffix".
        """
        return f"{self.major}.{self.minor}.{self.patch}{self.suffix}"


def data() -> dict:
    """
    Loads the program's data from data/data.json

    :return: Dictionary (JSON) containing program data
    """
    # Get the absolute path of the current file
    current_dir = os.path.dirname(os.path.abspath(__file__))

    # Construct the path to the data.json file
    settings_path = os.path.join(current_dir, "data", "data.json")

    # Load the settings from the file
    with open(settings_path) as file:
        program_data = json.load(file)

    return program_data


# Create parser
def create_parser():
    parser = argparse.ArgumentParser(description=f"{data()['program']['name']} v{Version().full_version()} — by" 
                                                 f" {data()['program']['developer']['name']}" 
                                                 f" ({data()['program']['developer']['about']})",
                                     epilog=data()['program']['about'])
    parser.add_argument("query", help="query string or text file containing queries")
    parser.add_argument("-c", "--count", help="number of results to show (default %(default)s).", default=10)
    parser.add_argument("-o", "--output", help="write output to specified file.")
    parser.add_argument("-v", "--version", action="version",
                        version=Version().full_version())
    return parser


logging.basicConfig(level="NOTSET", format="%(message)s",
                    handlers=[RichHandler(markup=True, log_time_format='%I:%M:%S%p', show_level=False)])
log = logging.getLogger("rich")



import os
import json
from googlesearch import search
from rich import print as xprint
from urllib.request import urlopen
from oxdork.config import log, Version

version = Version()


def process_input(user_input) -> list:
    """
    Processes user input. If input is a file, opens the file and reads the contents, line by line.
    If not, use the input as it is

    :param user_input: user input
    :return: A list of the processed queries/query
    """
    if os.path.isfile(user_input):
        with open(user_input, 'r') as file:
            log.info(f"Loaded queries from file: {file.name}")
            queries = file.readlines()
            queries = [query.strip() for query in queries]
            return queries
    else:
        return [user_input]


def begin_search(query, count, output) -> None:
    """
    Start search with the processed input

    :param query: Search query
    :param count: Number of results to return (defaults to 10)
    :param output: String representing a file to which results will be written
    :return: None
    """
    number = 0
    log.info(f"Fetching {count} results for `{query}`...")
    for counter, result in enumerate(search(query, num=int(count), start=0, stop=None,
                                            lang="en", tld="com", pause=2.5), start=1):
        number += 1
        log.info(f"{counter}. {result}")

        # Start writing results to a file
        # if user passes the filename to -o/--output
        if output:
            __write_output(result, counter, output)

        # If result number is greate than or equal to
        # the user specified count limit. Break the search loop. 
        if number >= int(count):
            break


def __write_output(result: str, counter: int, filename: str) -> None:
    """
    Write a result to a file.

    :param result: The result to write.
    :type result: str
    :param counter: The index of the result.
    :type counter: int
    :param filename: The name of the output file (without extension).
    :type filename: str
    :return: None
    """
    with open(f"{filename}.txt", "a") as file:
        file.write(f"{counter}. {result}\n")


def check_updates() -> None:
    """
    Checks for latest updates by retrieving the release tag from the releases page of the program from GitHub
    Then compares the remote version tag with the tag in the program.
    If they match, program assumes it's up-to-date.
    If not, print a message notifying the user about the remote version (which is treated as the official new release)
    , and lastly prints the release notes of the presumed new release.

    :return: None
    """
    with urlopen("https://api.github.com/repos/rly0nheart/oxdork/releases/latest") as response:
        release_data = json.loads(response.read().decode())
        if release_data['tag_name'] != version.full_version():
            xprint(f"* A new release of `OxDork` is available ({release_data['tag_name']}).\n")
        else:
            pass



import time
from oxdork.oxdork import *
from datetime import datetime
from oxdork.config import data, create_parser


parser = create_parser()
args = parser.parse_args()


def run():
    start_time = datetime.now()
    log.info(f"Starting {data()['program']['name']} v{version.full_version()} at {time.asctime()}")
    try:
        check_updates()
        queries = process_input(args.query)
        for query in queries:
            begin_search(query, count=args.count, output=args.output)
    except KeyboardInterrupt as ctrlc:
        log.warning(f"User interruption detected: {ctrlc}")
    except Exception as error:
        log.error(f"An error occurred: {error}")
    finally:
        log.info(f"Finished in {datetime.now()-start_time} seconds.")
        
